"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
function toSeverity(severity) {
    switch (severity) {
        case 'warning': return vscode_1.DiagnosticSeverity.Warning;
        case 'error': return vscode_1.DiagnosticSeverity.Error;
        case 'information': return vscode_1.DiagnosticSeverity.Information;
        default: return vscode_1.DiagnosticSeverity.Error;
    }
}
class LeanDiagnosticsProvider {
    constructor(server) {
        this.server = server;
        this.subscriptions = [];
        this.collection = vscode_1.languages.createDiagnosticCollection('lean');
        this.subscriptions.push(this.collection);
        this.subscriptions.push(server.allMessages.on((msgs) => this.updateDiagnostics(msgs.msgs)));
        this.subscriptions.push(server.restarted.on(() => this.updateDiagnostics([])));
    }
    updateDiagnostics(messages) {
        const diagnosticMap = new Map();
        for (const message of messages) {
            const pos = new vscode_1.Position(message.pos_line - 1, message.pos_col);
            const range = new vscode_1.Range(pos, pos);
            let diagnostics = diagnosticMap.get(message.file_name);
            if (!diagnostics) {
                diagnosticMap.set(message.file_name, diagnostics = []);
            }
            const d = new vscode_1.Diagnostic(range, message.text, toSeverity(message.severity));
            d.source = 'Lean';
            diagnostics.push(d);
        }
        const fileNameToUri = new Map();
        for (const doc of vscode_1.workspace.textDocuments) {
            fileNameToUri.set(doc.fileName, doc.uri);
        }
        this.collection.clear();
        diagnosticMap.forEach((diags, file) => this.collection.set(fileNameToUri.get(file), diags));
    }
    dispose() {
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.LeanDiagnosticsProvider = LeanDiagnosticsProvider;
//# sourceMappingURL=diagnostics.js.map