"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
// tslint:disable-next-line:no-var-requires
const cheerio = require('cheerio');
const url_1 = require("url");
const vscode_1 = require("vscode");
function mkCommandUri(commandName, ...args) {
    return `command:${commandName}?${encodeURIComponent(JSON.stringify(args))}`;
}
exports.mkCommandUri = mkCommandUri;
class DocViewProvider {
    constructor() {
        this.subscriptions = [];
        this.currentURL = undefined;
        this.backstack = [];
        this.forwardstack = [];
        this.subscriptions.push(vscode_1.commands.registerCommand('lean.openDocView', (url) => this.open(url)), vscode_1.commands.registerCommand('lean.backDocView', () => this.back()), vscode_1.commands.registerCommand('lean.forwardDocView', () => this.forward()), vscode_1.commands.registerCommand('lean.openTryIt', (code) => this.tryIt(code)));
    }
    tryIt(code) {
        return __awaiter(this, void 0, void 0, function* () {
            const doc = yield vscode_1.workspace.openTextDocument({ language: 'lean', content: code });
            const editor = yield vscode_1.window.showTextDocument(doc, vscode_1.ViewColumn.One);
        });
    }
    getWebview() {
        if (!this.webview) {
            this.webview = vscode_1.window.createWebviewPanel('lean', 'Lean Documentation', { viewColumn: 3 }, { enableFindWidget: true, enableScripts: true, enableCommandUris: true });
            this.webview.onDidDispose(() => this.webview = null);
        }
        return this.webview;
    }
    fetch(url) {
        return __awaiter(this, void 0, void 0, function* () {
            if (url) {
                return (yield axios_1.default.get(url)).data;
            }
            else {
                const books = {
                    'Theorem Proving in Lean': 'https://leanprover.github.io/theorem_proving_in_lean/',
                    'Reference Manual': 'https://leanprover.github.io/reference/',
                };
                return '<ul>' +
                    Object.getOwnPropertyNames(books).map((n) => `<li><a href="${books[n]}">${n}</a></li>`).join('') +
                    '</ul>';
            }
        });
    }
    setHtml() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = this.currentURL;
            const $ = cheerio.load(yield this.fetch(url));
            for (const style of $('link[rel=stylesheet]').get()) {
                style.attribs.href = new url_1.URL(style.attribs.href, url).toString();
            }
            for (const script of $('script[src]').get()) {
                script.attribs.src = new url_1.URL(script.attribs.src, url).toString();
            }
            for (const link of $('a[href]').get()) {
                const tryItMatch = link.attribs.href.match(/\/live\/.*#code=(.*)/);
                if (tryItMatch) {
                    const code = decodeURIComponent(tryItMatch[1]);
                    link.attribs.href = mkCommandUri('lean.openTryIt', code);
                }
                else {
                    const href = new url_1.URL(link.attribs.href, url).toString();
                    link.attribs.href = mkCommandUri('lean.openDocView', href);
                }
            }
            $(`<nav style="
                width:100vw; position : fixed; top : 0px; left : 0px;
                padding : 4px; background : #f3f3f3; z-index:100
              ">
            <a href="${mkCommandUri('lean.backDocView')}" title="back">← back</a>
            <a href="${mkCommandUri('lean.forwardDocView')}" title="forward">→ forward</a>
        </nav>`).prependTo('body');
            $('nav+*').css('margin-top', '3em');
            this.getWebview().webview.html = $.html();
        });
    }
    /** Called by the user clicking a link. */
    open(url) {
        return __awaiter(this, void 0, void 0, function* () {
            if (url) {
                this.backstack.push(this.currentURL);
                this.forwardstack = [];
            }
            this.currentURL = url;
            yield this.setHtml();
        });
    }
    back() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.backstack.length === 0) {
                return;
            }
            this.forwardstack.push(this.currentURL);
            this.currentURL = this.backstack.pop();
            yield this.setHtml();
        });
    }
    forward() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.forwardstack.length === 0) {
                return;
            }
            this.backstack.push(this.currentURL);
            this.currentURL = this.forwardstack.pop();
            yield this.setHtml();
        });
    }
    dispose() {
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.DocViewProvider = DocViewProvider;
//# sourceMappingURL=docview.js.map