"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const loadJsonFile = require("load-json-file");
const vscode_1 = require("vscode");
const batch_1 = require("./batch");
const completion_1 = require("./completion");
const definition_1 = require("./definition");
const diagnostics_1 = require("./diagnostics");
const docview_1 = require("./docview");
const holes_1 = require("./holes");
const hover_1 = require("./hover");
const infoview_1 = require("./infoview");
const input_1 = require("./input");
const leanpkg_1 = require("./leanpkg");
const roi_1 = require("./roi");
const search_1 = require("./search");
const server_1 = require("./server");
const statusbar_1 = require("./statusbar");
const sync_1 = require("./sync");
const taskgutter_1 = require("./taskgutter");
// Seeing .olean files in the source tree is annoying, we should
// just globally hide them.
function configExcludeOLean() {
    const files = vscode_1.workspace.getConfiguration('files');
    const exclude = files.get('exclude');
    exclude['**/*.olean'] = true;
    files.update('exclude', exclude, true);
}
const LEAN_MODE = {
    language: 'lean',
};
function activate(context) {
    configExcludeOLean();
    const server = new server_1.Server();
    context.subscriptions.push(server);
    const roiManager = new roi_1.RoiManager(server, LEAN_MODE);
    context.subscriptions.push(roiManager);
    // The sync service starts automatically starts
    // the server when it sees a *.lean file.
    context.subscriptions.push(new sync_1.LeanSyncService(server, LEAN_MODE));
    // Setup the commands.
    context.subscriptions.push(vscode_1.commands.registerCommand('lean.restartServer', () => server.restart()), vscode_1.commands.registerTextEditorCommand('lean.batchExecute', (editor, edit, args) => { batch_1.batchExecuteFile(server, editor, edit, args); }));
    context.subscriptions.push(new diagnostics_1.LeanDiagnosticsProvider(server));
    // Task messages.
    context.subscriptions.push(new taskgutter_1.LeanTaskGutter(server, context), new taskgutter_1.LeanTaskMessages(server));
    // Register the support for hovering.
    context.subscriptions.push(vscode_1.languages.registerHoverProvider(LEAN_MODE, new hover_1.LeanHoverProvider(server)));
    // Register support for completion.
    context.subscriptions.push(vscode_1.languages.registerCompletionItemProvider(LEAN_MODE, new completion_1.LeanCompletionItemProvider(server), '.'));
    // Register support for unicode input.
    (() => __awaiter(this, void 0, void 0, function* () {
        const translations = yield loadJsonFile(context.asAbsolutePath('translations.json'));
        const inputLanguages = input_1.inputModeLanguages();
        const hoverProvider = vscode_1.languages.registerHoverProvider(inputLanguages, new input_1.LeanInputExplanationHover(translations));
        context.subscriptions.push(hoverProvider, new input_1.LeanInputAbbreviator(translations));
    }))();
    // Load the language-configuration manually, so that we can set the wordPattern.
    let langConf = vscode_1.Uri.file(context.asAbsolutePath('language-configuration.json')).toJSON();
    langConf = { comments: langConf.comments, brackets: langConf.brackets,
        autoClosingPairs: langConf.autoClosingPairs, surroundingPairs: langConf.surroundingPairs,
        wordPattern: /(-?\d*\.\d\w*)|([^`~!@$%^&*()-=+\[{\]}\\|;:",./?\s]+)/ };
    context.subscriptions.push(vscode_1.languages.setLanguageConfiguration('lean', langConf));
    // Register support for definition support.
    context.subscriptions.push(vscode_1.languages.registerDefinitionProvider(LEAN_MODE, new definition_1.LeanDefinitionProvider(server)));
    // Search
    context.subscriptions.push(vscode_1.languages.registerWorkspaceSymbolProvider(new search_1.LeanWorkspaceSymbolProvider(server)));
    // Holes
    context.subscriptions.push(new holes_1.LeanHoles(server, LEAN_MODE));
    // Add item to the status bar.
    context.subscriptions.push(new statusbar_1.LeanStatusBarItem(server, roiManager));
    // Add info view: listing either the current goal state or a list of all error messages
    context.subscriptions.push(new infoview_1.InfoProvider(server, LEAN_MODE, context));
    context.subscriptions.push(new leanpkg_1.LeanpkgService(server));
    context.subscriptions.push(new docview_1.DocViewProvider());
}
exports.activate = activate;
//# sourceMappingURL=extension.js.map