"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const vscode_1 = require("vscode");
function compareMessages(m1, m2) {
    return (m1.file_name === m2.file_name &&
        m1.pos_line === m2.pos_line && m1.pos_col === m2.pos_col &&
        m1.severity === m2.severity && m1.caption === m2.caption && m1.text === m2.text);
}
// https://stackoverflow.com/questions/6234773/can-i-escape-html-special-chars-in-javascript
function escapeHtml(s) {
    return s
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&#039;');
}
var DisplayMode;
(function (DisplayMode) {
    DisplayMode[DisplayMode["OnlyState"] = 0] = "OnlyState";
    DisplayMode[DisplayMode["AllMessage"] = 1] = "AllMessage";
})(DisplayMode || (DisplayMode = {}));
class InfoProvider {
    constructor(server, leanDocs, context) {
        this.server = server;
        this.leanDocs = leanDocs;
        this.context = context;
        this.subscriptions = [];
        this.displayMode = DisplayMode.AllMessage;
        this.statusShown = false;
        this.started = false;
        this.stopped = false;
        this.curFileName = null;
        this.curPosition = null;
        this.curGoalState = null;
        this.curMessages = null;
        this.stylesheet = null;
        this.statusBarItem = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Right, 1000);
        this.hoverDecorationType = vscode_1.window.createTextEditorDecorationType({
            backgroundColor: 'red',
            border: '3px solid red',
        });
        this.updateStylesheet();
        this.subscriptions.push(this.server.allMessages.on(() => {
            if (this.updateMessages()) {
                this.rerender();
            }
        }), this.server.statusChanged.on(() => __awaiter(this, void 0, void 0, function* () {
            if (this.displayMode === DisplayMode.OnlyState) {
                const changed = yield this.updateGoal();
                if (changed) {
                    this.rerender();
                }
            }
        })), this.server.restarted.on(() => {
            this.autoOpen();
        }), vscode_1.window.onDidChangeActiveTextEditor(() => this.updatePosition(false)), vscode_1.window.onDidChangeTextEditorSelection(() => this.updatePosition(false)), vscode_1.workspace.onDidChangeConfiguration((e) => {
            this.updateStylesheet();
            this.rerender();
            if (!vscode_1.workspace.getConfiguration('lean').get('typeInStatusBar') && this.statusShown) {
                this.statusBarItem.hide();
                this.statusShown = false;
            }
        }), vscode_1.commands.registerCommand('_lean.revealPosition', this.revealEditorPosition), vscode_1.commands.registerCommand('_lean.infoView.pause', () => {
            this.stopUpdating();
        }), vscode_1.commands.registerCommand('_lean.infoView.continue', () => {
            this.setMode(this.displayMode);
        }), vscode_1.commands.registerTextEditorCommand('lean.displayGoal', (editor) => {
            this.setMode(DisplayMode.OnlyState);
            this.openPreview(editor);
        }), vscode_1.commands.registerTextEditorCommand('lean.displayList', (editor) => {
            this.setMode(DisplayMode.AllMessage);
            this.openPreview(editor);
        }), vscode_1.commands.registerTextEditorCommand('lean.infoView.displayGoal', (editor) => {
            this.setMode(DisplayMode.OnlyState);
        }), vscode_1.commands.registerTextEditorCommand('lean.infoView.displayList', (editor) => {
            this.setMode(DisplayMode.AllMessage);
        }), vscode_1.commands.registerTextEditorCommand('lean.infoView.copyToComment', (editor) => this.copyToComment(editor)), vscode_1.commands.registerTextEditorCommand('lean.infoView.toggleUpdating', (editor) => {
            if (this.stopped) {
                this.setMode(this.displayMode);
            }
            else {
                this.stopUpdating();
            }
        }));
        if (this.server.alive()) {
            this.autoOpen();
        }
    }
    dispose() {
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
    updateStylesheet() {
        const css = this.context.asAbsolutePath(path_1.join('media', `infoview.css`));
        const fontFamily = vscode_1.workspace.getConfiguration('editor').get('fontFamily').
            replace(/['"]/g, '');
        this.stylesheet = fs_1.readFileSync(css, 'utf-8') + `
            pre {
                font-family: ${fontFamily};
                font-size: ${vscode_1.workspace.getConfiguration('editor').get('fontSize')}px;
                white-space: pre-wrap; // TODO(gabriel): make configurable
            }
            ` +
            vscode_1.workspace.getConfiguration('lean').get('infoViewStyle');
    }
    autoOpen() {
        if (!this.started && vscode_1.workspace.getConfiguration('lean').get('infoViewAutoOpen')) {
            this.started = true;
            this.setMode(vscode_1.workspace.getConfiguration('lean').get('infoViewAutoOpenShowGoal', true) ?
                DisplayMode.OnlyState : DisplayMode.AllMessage);
            this.openPreview(vscode_1.window.activeTextEditor);
            this.updatePosition(false);
        }
    }
    openPreview(editor) {
        let column = editor ? editor.viewColumn + 1 : vscode_1.ViewColumn.Two;
        if (column === 4) {
            column = vscode_1.ViewColumn.Three;
        }
        if (this.webviewPanel) {
            this.webviewPanel.reveal(column, true);
        }
        else {
            this.webviewPanel = vscode_1.window.createWebviewPanel('lean', 'Lean Messages', { viewColumn: column, preserveFocus: true }, {
                enableFindWidget: true,
                retainContextWhenHidden: true,
                enableScripts: true,
                enableCommandUris: true,
            });
            this.webviewPanel.webview.html = this.render();
            this.webviewPanel.onDidDispose(() => this.webviewPanel = null);
            this.webviewPanel.webview.onDidReceiveMessage((message) => {
                switch (message.command) {
                    case 'selectFilter':
                        vscode_1.workspace.getConfiguration('lean').update('infoViewFilterIndex', message.filterId, true);
                        // the workspace configuration change already forces a rerender
                        return;
                    case 'hoverPosition':
                        this.hoverEditorPosition(message.data[0], message.data[1], message.data[2], message.data[3], message.data[4]);
                        return;
                    case 'stopHover':
                        this.stopHover();
                        return;
                }
            }, undefined, this.subscriptions);
        }
    }
    sendPosition() {
        this.postMessage({
            command: 'position',
            fileName: this.curFileName,
            line: this.curPosition.line + 1,
            column: this.curPosition.character,
        });
    }
    stopUpdating() {
        this.stopped = true;
        this.postMessage({ command: 'pause' });
    }
    setMode(mode) {
        if (this.displayMode === mode && !this.stopped) {
            return;
        }
        this.displayMode = mode;
        this.stopped = false;
        this.updatePosition(true);
    }
    rerender() {
        if (this.webviewPanel) {
            this.webviewPanel.webview.html = this.render();
            this.stopHover();
        }
    }
    postMessage(msg) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.webviewPanel) {
                return this.webviewPanel.webview.postMessage(msg);
            }
            else {
                return false;
            }
        });
    }
    revealEditorPosition(uri, line, column) {
        for (const editor of vscode_1.window.visibleTextEditors) {
            if (editor.document.uri.toString() === uri.toString()) {
                const pos = new vscode_1.Position(line - 1, column);
                editor.revealRange(new vscode_1.Range(pos, pos), vscode_1.TextEditorRevealType.InCenterIfOutsideViewport);
                editor.selection = new vscode_1.Selection(pos, pos);
            }
        }
    }
    hoverEditorPosition(uri, line, column, endLine, endColumn) {
        for (const editor of vscode_1.window.visibleTextEditors) {
            if (editor.document.uri.toString() === uri) {
                const pos = new vscode_1.Position(line - 1, column);
                const endPos = new vscode_1.Position(endLine - 1, endColumn);
                const range = new vscode_1.Range(pos, endPos);
                editor.setDecorations(this.hoverDecorationType, [range]);
            }
        }
    }
    stopHover() {
        for (const editor of vscode_1.window.visibleTextEditors) {
            if (editor.document.languageId === 'lean') {
                editor.setDecorations(this.hoverDecorationType, []);
            }
        }
    }
    changePosition() {
        if (!vscode_1.window.activeTextEditor ||
            !vscode_1.languages.match(this.leanDocs, vscode_1.window.activeTextEditor.document)) {
            return;
        }
        const oldFileName = this.curFileName;
        const oldPosition = this.curPosition;
        this.curFileName = vscode_1.window.activeTextEditor.document.fileName;
        this.curPosition = vscode_1.window.activeTextEditor.selection.active;
        return (this.curFileName !== oldFileName || !this.curPosition.isEqual(oldPosition));
    }
    updatePosition(forceRefresh) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stopped) {
                return;
            }
            const chPos = this.changePosition();
            if (!chPos && !forceRefresh) {
                return;
            }
            // clear type in status bar item
            this.statusBarItem.text = '';
            const chMsg = this.updateMessages();
            /* updateTypeStatus is only called from the cases of the following switch-block, so pausing
               live-updates to the infoview (via this.stopped) also pauses the type status bar item */
            switch (this.displayMode) {
                case DisplayMode.OnlyState:
                    const chGoal = yield this.updateGoal();
                    if (chPos || chGoal || chMsg) {
                        this.rerender();
                    }
                    else if (forceRefresh) {
                        this.postMessage({ command: 'continue' });
                    }
                    break;
                case DisplayMode.AllMessage:
                    if (vscode_1.workspace.getConfiguration('lean').get('typeInStatusBar')) {
                        const info = yield this.server.info(this.curFileName, this.curPosition.line + 1, this.curPosition.character);
                        this.updateTypeStatus(info);
                    }
                    if (forceRefresh || chMsg) {
                        this.rerender();
                    }
                    else {
                        this.sendPosition();
                    }
                    break;
            }
        });
    }
    updateMessages() {
        if (this.stopped || !this.curFileName) {
            return false;
        }
        let msgs;
        switch (this.displayMode) {
            case DisplayMode.OnlyState:
                /* Heuristic: find first position to the left which has messages attached,
                   from that on show all messages in this line */
                msgs = this.server.messages
                    .filter((m) => m.file_name === this.curFileName &&
                    m.pos_line === this.curPosition.line + 1)
                    .sort((a, b) => a.pos_col - b.pos_col);
                if (!vscode_1.workspace.getConfiguration('lean').get('infoViewAllErrorsOnLine')) {
                    let startColumn;
                    let startPos = null;
                    for (let i = 0; i < msgs.length; i++) {
                        if (this.curPosition.character < msgs[i].pos_col) {
                            break;
                        }
                        if (this.curPosition.character === msgs[i].pos_col) {
                            startColumn = this.curPosition.character;
                            startPos = i;
                            break;
                        }
                        if (startColumn == null || startColumn < msgs[i].pos_col) {
                            startColumn = msgs[i].pos_col;
                            startPos = i;
                        }
                    }
                    if (startPos) {
                        msgs = msgs.slice(startPos);
                    }
                }
                break;
            case DisplayMode.AllMessage:
                msgs = this.server.messages
                    .filter((m) => m.file_name === this.curFileName)
                    .sort((a, b) => a.pos_line === b.pos_line
                    ? a.pos_col - b.pos_col
                    : a.pos_line - b.pos_line);
                break;
        }
        if (!this.curMessages) {
            this.curMessages = msgs;
            return true;
        }
        const oldMsgs = this.curMessages;
        if (msgs.length === oldMsgs.length) {
            let eq = true;
            for (let i = 0; i < msgs.length; i++) {
                if (!compareMessages(msgs[i], oldMsgs[i])) {
                    eq = false;
                    break;
                }
            }
            if (eq) {
                return false;
            }
        }
        this.curMessages = msgs;
        return true;
    }
    updateGoal() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stopped) {
                return false;
            }
            const info = yield this.server.info(this.curFileName, this.curPosition.line + 1, this.curPosition.character);
            if (info.record && info.record.state) {
                if (this.curGoalState !== info.record.state) {
                    this.curGoalState = info.record.state;
                    return true;
                }
            }
            else {
                if (this.curGoalState) {
                    this.curGoalState = null;
                    return false;
                }
            }
            if (vscode_1.workspace.getConfiguration('lean').get('typeInStatusBar')) {
                this.updateTypeStatus(info);
            }
        });
    }
    updateTypeStatus(info) {
        if (!this.statusShown) {
            this.statusBarItem.show();
            this.statusShown = true;
        }
        if (info.record) {
            const name = info.record['full-id'] || info.record.text;
            if (name && !info.record.tactic_params) {
                this.statusBarItem.text = name + ' : ' + info.record.type;
            }
        }
    }
    getMediaPath(mediaFile) {
        return vscode_1.Uri.file(this.context.asAbsolutePath(path_1.join('media', mediaFile)))
            .with({ scheme: 'vscode-resource' }).toString();
    }
    render() {
        const header = `<!DOCTYPE html>
            <html>
            <head>
                <meta http-equiv="Content-type" content="text/html;charset=utf-8">
                <style>${this.stylesheet}</style>
                <script charset="utf-8" src="${this.getMediaPath('infoview-ctrl.js')}"></script>
            </head>`;
        if (!this.curFileName) {
            return header + '<body>No Lean file active</body>';
        }
        return header +
            `<body
                data-uri="${encodeURI(vscode_1.Uri.file(this.curFileName).toString())}"
                data-line="${(this.curPosition.line + 1).toString()}"
                data-column="${this.curPosition.character.toString()}"
                ${this.displayMode === DisplayMode.AllMessage ? "data-messages=''" : ''}>
                <div id="debug"></div>
                ${this.curGoalState ? this.renderFilter() : ''}
                <div id="run-state">
                    <span id="state-continue">Stopped <a href="command:_lean.infoView.continue?{}">
                        <img title="Continue Updating" src="${this.getMediaPath('continue.svg')}"></a></span>
                    <span id="state-pause">Updating <a href="command:_lean.infoView.pause?{}">
                        <img title="Stop Updating" src="${this.getMediaPath('pause.svg')}"></span></a>
                </div>
                ${this.renderGoal()}
                <div id="messages">${this.renderMessages()}</div>
            </body></html>`;
    }
    renderFilter() {
        const reFilters = vscode_1.workspace.getConfiguration('lean').get('infoViewTacticStateFilters', []);
        const filterIndex = vscode_1.workspace.getConfiguration('lean').get('infoViewFilterIndex', -1);
        return reFilters.length > 0 ?
            `<div id="filter">
            <select id="filterSelect" onchange="infoViewModule.selectFilter(this.value);">
                <option value="-1" ${filterIndex === -1 ? 'selected' : ''}>no filter</option>
                ${reFilters.map((obj, i) => `<option value="${i}" ${filterIndex === i ? 'selected' : ''}>
                    ${obj.name ||
                `${obj.match ? 'show ' : 'hide '}/${obj.regex}/${obj.flags}`}</option>`)}
            </select>
        </div>` : '';
    }
    colorizeMessage(goal) {
        return escapeHtml(goal)
            .replace(/^([|⊢]) /mg, '<strong class="goal-vdash">$1</strong> ')
            .replace(/^(\d+ goals|1 goal)/mg, '<strong class="goal-goals">$1</strong>')
            .replace(/^(context|state):/mg, '<strong class="goal-goals">$1</strong>:')
            .replace(/^(case) /mg, '<strong class="goal-case">$1</strong> ')
            .replace(/^([^:\n< ][^:\n⊢{[(⦃]*) :/mg, '<strong class="goal-hyp">$1</strong> :');
    }
    renderGoal() {
        if (!this.curGoalState || this.displayMode !== DisplayMode.OnlyState) {
            return '';
        }
        const reFilters = vscode_1.workspace.getConfiguration('lean').get('infoViewTacticStateFilters', []);
        const filterIndex = vscode_1.workspace.getConfiguration('lean').get('infoViewFilterIndex', -1);
        let goalString = this.curGoalState.replace(/^(no goals)/mg, 'goals accomplished');
        goalString = RegExp('^\\d+ goals|goals accomplished', 'mg').test(goalString) ? goalString :
            '1 goal\n'.concat(goalString);
        const filteredGoalState = reFilters.length === 0 || filterIndex === -1 ? goalString :
            // this regex splits the goal state into (possibly multi-line) hypothesis and goal blocks
            // by keeping indented lines with the most recent non-indented line
            goalString.match(/(^(?!  ).*\n?(  .*\n?)*)/mg).map((line) => line.trim())
                .filter((line) => {
                const filt = reFilters[filterIndex];
                const test = line.match(new RegExp(filt.regex, filt.flags)) !== null;
                return filt.match ? test : !test;
            }).join('\n');
        return `<div id="goal"><h1>Tactic State</h1><pre>${this.colorizeMessage(filteredGoalState)}</pre></div>`;
    }
    renderMessages() {
        if (!this.curFileName || !this.curMessages) {
            return ``;
        }
        return this.curMessages.map((m) => {
            const f = escapeHtml(m.file_name);
            const b = escapeHtml(path_1.basename(m.file_name));
            const l = m.pos_line;
            const c = m.pos_col;
            const el = m.end_pos_line || l;
            const ec = m.end_pos_col || c;
            const cmd = encodeURI('command:_lean.revealPosition?' +
                JSON.stringify([vscode_1.Uri.file(m.file_name), m.pos_line, m.pos_col]));
            const shouldColorize = m.severity === 'error';
            const colorized = shouldColorize ? this.colorizeMessage(m.text) :
                escapeHtml(m.text);
            return `<div class="message ${m.severity}" data-line="${l}" data-column="${c}"
                data-end-line="${el}" data-end-column="${ec}">
                <h1 title="${f}:${l}:${c}"><a href="${cmd}">
                    ${b}:${l}:${c}: ${m.severity} ${escapeHtml(m.caption)}
                </a></h1>
                <pre>${colorized}</pre></div>`;
        }).join('\n');
    }
    copyToComment(editor) {
        return __awaiter(this, void 0, void 0, function* () {
            yield editor.edit((builder) => {
                builder.insert(editor.selection.end.with({ character: 0 }).translate({ lineDelta: 1 }), '/-\n' + this.renderText() + '\n-/\n');
            });
        });
    }
    renderText() {
        const msgText = this.curMessages &&
            this.curMessages.map((m) => `${path_1.basename(m.file_name)}:${m.pos_line}:${m.pos_col}: ${m.severity} ${m.caption}\n${m.text}`).join('\n');
        const goalText = this.curGoalState ? `Tactic State:\n${this.curGoalState}\n` : '';
        return goalText + msgText;
    }
}
exports.InfoProvider = InfoProvider;
//# sourceMappingURL=infoview.js.map