"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const roi_1 = require("./roi");
class LeanStatusBarItem {
    constructor(server, roiManager) {
        this.server = server;
        this.roiManager = roiManager;
        this.shown = false;
        this.subscriptions = [];
        this.statusBarItem = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Left, 10);
        this.subscriptions.push(server.restarted.on(() => this.update()), server.statusChanged.on(() => this.update()));
        if (roiManager) {
            this.subscriptions.push(roiManager.onModeChanged(() => this.update()));
            this.statusBarItem.command = 'lean.roiMode.select';
        }
        if (this.server.alive()) {
            this.update();
        }
    }
    update() {
        let text = 'Lean: ';
        const serverStatus = this.server.statusChanged.currentValue;
        if (serverStatus && serverStatus.isRunning) {
            text += `$(sync) ${serverStatus.numberOfTasks}`;
        }
        else if (serverStatus && serverStatus.stopped) {
            text += '$(x)';
        }
        else {
            text += '$(check)';
        }
        switch (this.roiManager && this.roiManager.mode) {
            case roi_1.RoiMode.Nothing:
                text += ' (checking nothing)';
                break;
            case roi_1.RoiMode.VisibleLines:
                text += ' (checking visible lines)';
                break;
            case roi_1.RoiMode.VisibleLinesAndAbove:
                text += ' (checking visible lines and above)';
                break;
            case roi_1.RoiMode.VisibleFiles:
                text += ' (checking visible files)';
                break;
            case roi_1.RoiMode.OpenFiles:
                text += ' (checking open files)';
                break;
            case roi_1.RoiMode.ProjectFiles:
                text += ' (checking project files)';
                break;
        }
        if (!this.shown) {
            this.statusBarItem.show();
            this.shown = true;
        }
        this.statusBarItem.text = text;
    }
    dispose() {
        this.statusBarItem.dispose();
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.LeanStatusBarItem = LeanStatusBarItem;
//# sourceMappingURL=statusbar.js.map