"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
class LeanSyncService {
    constructor(server, documentFilter) {
        this.server = server;
        this.documentFilter = documentFilter;
        this.subscriptions = [];
        // The sync service starts automatically starts
        // the server when it sees a *.lean file.
        this.didAutoStartServer = false;
        this.subscriptions.push(vscode_1.workspace.onDidChangeTextDocument((ev) => this.syncDoc(ev.document)));
        this.subscriptions.push(vscode_1.workspace.onDidOpenTextDocument((ev) => this.syncDoc(ev)));
        // TODO(gabriel): unload files from server
        // this.subscriptions.push(workspace.onDidCloseTextDocument(???));
        // Sync files that are already open.
        this.syncAll();
        this.subscriptions.push(this.server.restarted.on(() => this.syncAll()));
    }
    syncAll() {
        vscode_1.workspace.textDocuments.forEach((doc) => this.syncDoc(doc));
    }
    syncDoc(doc) {
        if (!vscode_1.languages.match(this.documentFilter, doc)) {
            return;
        }
        if (!this.didAutoStartServer && !this.server.alive()) {
            this.didAutoStartServer = true;
            this.server.connect();
        }
        this.server.sync(doc.fileName, doc.getText());
    }
    dispose() {
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.LeanSyncService = LeanSyncService;
//# sourceMappingURL=sync.js.map