"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
class LeanTaskGutter {
    constructor(server, context) {
        this.subscriptions = [];
        this.decoration = vscode_1.window.createTextEditorDecorationType({
            overviewRulerLane: vscode_1.OverviewRulerLane.Left,
            overviewRulerColor: 'rgba(255, 165, 0, 0.5)',
            dark: {
                gutterIconPath: context.asAbsolutePath('media/progress-dark.svg'),
            },
            light: {
                gutterIconPath: context.asAbsolutePath('media/progress-light.svg'),
            },
            gutterIconSize: 'contain',
        });
        this.subscriptions.push(server.statusChanged.on((status) => this.updateDecos(status)));
    }
    updateDecos(status) {
        for (const editor of vscode_1.window.visibleTextEditors) {
            editor.setDecorations(this.decoration, status.tasks.filter((t) => t.file_name === editor.document.fileName)
                .map((task) => ({
                range: new vscode_1.Range(task.pos_line - 1, task.pos_col, task.end_pos_line - 1, task.end_pos_col),
                hoverMessage: task.desc,
            })));
        }
    }
    dispose() {
        this.decoration.dispose();
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.LeanTaskGutter = LeanTaskGutter;
class LeanTaskMessages {
    constructor(server) {
        this.subscriptions = [];
        this.collection = vscode_1.languages.createDiagnosticCollection('lean-tasks');
        this.subscriptions.push(server.statusChanged.on((status) => this.updateMsgs(status)));
        this.subscriptions.push(vscode_1.workspace.onDidChangeConfiguration(() => this.updateMsgs(server.statusChanged.currentValue)));
    }
    updateMsgs(status) {
        const diagsPerFile = new Map();
        if (vscode_1.workspace.getConfiguration('lean').get('progressMessages')) {
            for (const task of status.tasks) {
                let diags = diagsPerFile.get(task.file_name);
                if (!diags) {
                    diagsPerFile.set(task.file_name, diags = []);
                }
                diags.push(new vscode_1.Diagnostic(new vscode_1.Range(task.pos_line - 1, task.pos_col, task.end_pos_line - 1, task.end_pos_col), task.desc, vscode_1.DiagnosticSeverity.Information));
            }
        }
        this.collection.clear();
        diagsPerFile.forEach((diags, file) => this.collection.set(vscode_1.Uri.file(file), diags));
    }
    dispose() {
        this.collection.dispose();
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.LeanTaskMessages = LeanTaskMessages;
//# sourceMappingURL=taskgutter.js.map