"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
class LeanTypeStatusItem {
    constructor(server) {
        this.server = server;
        this.shown = false;
        this.subscriptions = [];
        this.statusBarItem = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Right, 100);
        this.subscriptions.push(server.restarted.on(() => this.update()), server.statusChanged.on(() => this.update()), vscode_1.window.onDidChangeActiveTextEditor(() => this.update()), vscode_1.window.onDidChangeTextEditorSelection(() => this.update()));
        if (this.server.alive()) {
            this.update();
        }
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            let text = 'Type: ';
            const fileName = vscode_1.window.activeTextEditor.document.fileName;
            const position = vscode_1.window.activeTextEditor.selection.active;
            console.log(fileName, position);
            try {
                const response = yield this.server.info(fileName, position.line + 1, position.character);
                console.log(response);
                if (response.record) {
                    const name = response.record['full-id'] || response.record.text;
                    if (name) {
                        if (!response.record.tactic_params) {
                            text += name + ' : ' + response.record.type;
                        }
                    }
                }
            }
            catch (e) {
                console.log(e);
            }
            if (!this.shown) {
                this.statusBarItem.show();
                this.shown = true;
            }
            this.statusBarItem.text = text;
        });
    }
    dispose() {
        this.statusBarItem.dispose();
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.LeanTypeStatusItem = LeanTypeStatusItem;
//# sourceMappingURL=typestatusitem.js.map